<?php
/**
 * Common use functions - files and dirs
 * @author José Carlos Cruz Parra AKA internia
 * josecarlos@programadorphpfreelance.com
 * http://programadorphpfreelance.com
 * 2008-2012
 * This code is released under the GNU General Public License.
 */




/**
 * @return mixed
 * @param $file string
 * @param $evalonerror string
 * @desc If given file exists, includes it, if not, evals given code string ("false" by default).
 */
function include_ifexists($file, $evalonerror="false;")
{
	return file_exists($file) ? include($file) : eval($evalonerror);
}
/**
 * @return mixed
 * @param $file string
 * @param $evalonerror string
 * @desc If given file exists, includes_once it, if not, evals given code string ("false" by default).
 */
function include_once_ifexists($file, $evalonerror="false")
{
	return file_exists($file) ? include_once($file) : eval($evalonerror);
}
/**
 * @return mixed
 * @param $file string
 * @param $evalonerror string
 * @desc If given file exists, requires it, if not, evals given code string ("false" by default).
 */
function require_ifexists($file, $evalonerror="false")
{
	return file_exists($file) ? require($file) : eval($evalonerror);
}
/**
 * @return mixed
 * @param $file string
 * @param $evalonerror string
 * @desc If given file exists, requires_once it, if not, evals given code string ("false" by default).
 */
function require_once_ifexists($file, $evalonerror="false")
{
	return file_exists($file) ? require_once($file) : eval($evalonerror);
}
/**
 * @return array
 * @param $dir string
 * @param $includepath bool
 * @param $recursive bool
 * @desc Gets the names of the dirs existing in a given dir, with dir path if $includepath is true.
 */
function getDirs($dir,$includepath=false,$recursive=false)
{
	$pos = strrpos($dir,'/');
	if($pos == (strlen($dir) - 1))
		$dir = substr($dir, 0, $pos);
	$currentdir = getcwd();
	chdir($dir);
	$handle = opendir(".");
	$dirs = array();
	while ($file = readdir($handle))
	{
		if(is_dir($file) && $file!="." && $file!="..")
		{
			$dirs[] = $includepath ? $dir."/".$file : $file;

			if($recursive)
			{
				if($includepath)
				{
					chdir($currentdir);
					$dirs = array_merge($dirs, getDirs($dir."/".$file,true,true));
					chdir($dir);
				}
				else
				{
					$dirs = array_merge($dirs, getDirs($file,false,true));
				}
			}
		}
	}
	chdir($currentdir);
	return $dirs;
}

function getSubDirs($dir)
{
	$len = mb_strlen($dir);
	$dirs = getDirs($dir, true, true);
	sort($dirs);
	for($c=0,$size=sizeof($dirs); $c<$size; $c++)
	{
		$dirs[$c] = mb_substr($dirs[$c], $len);
	}

	return $dirs;
}
/**
 * @return array
 * @param $dir string
 * @param $excludedextensions array
 * @param $includepath bool
 * @param $recursive bool
 * @desc Gets the names of the files existing in a given dir, with dir path if $includepath is true.
 */
function getFiles($dir,$excludedextensions=array(),$includepath=false,$recursive=false)
{
	$pos = strrpos($dir,'/');
	if($pos == (strlen($dir) - 1))
	{
		$dir = substr($dir, 0, $pos);
	}
	$currentdir = getcwd();
	chdir($dir);
	$handle = opendir(".");
	$files = array();
	while ($file = readdir($handle))
	{
		if(is_dir($file) && $file!="." && $file!=".." && $recursive)
		{
			if($includepath)
			{
				chdir($currentdir);
				$files = array_merge($files, getFiles($dir."/".$file,$excludedextensions,true,true));
				chdir($dir);
			}
			else
				$files = array_merge($files, getFiles($file,$excludedextensions,false,true));
		}
		elseif(is_file($file) && !in_array(getExtension($file),$excludedextensions))
		{
			if($includepath)
				$files[] = $dir."/".$file;
			else
				$files[] = $file;
		}
	}
	chdir($currentdir);
	return $files;
}
/**
 * @return array
 * @param $dir string
 * @param $includedextensions array
 * @param $includepath bool
 * @param $recursive bool
 * @desc Gets the names of the files existing in a given dir wich extension is in $includedextensions array, with dir path if $includepath is true.
 */
function getFilesExt($dir, $includedextensions, $includepath=false, $recursive=false)
{
	$pos = strrpos($dir,'/');
	if($pos == (strlen($dir) - 1))
	{
		$dir = substr($dir, 0, $pos);
	}
	$currentdir = getcwd();
	chdir($dir);
	$handle = opendir(".");
	$files = array();
	while(($file = readdir($handle)))
	{
		if(is_dir($file) && $file!="." && $file!=".." && $recursive)
		{
			if($includepath)
			{
				chdir($currentdir);
				$files = array_merge($files, getFilesExt($dir."/".$file,$includedextensions,true,true));
				chdir($dir);
			}
			else
			{
				$files = array_merge($files, getFilesExt($file,$includedextensions,false,true));
			}
		}
		elseif(is_file($file) && in_array(getExtension($file),$includedextensions))
		{
			$files[] = $includepath ? $dir."/".$file : $file;
		}
	}
	chdir($currentdir);

	return $files;
}

function getRandomFileExt($dir, $includedextensions, $includepath=false)
{
	$files = getFilesExt($dir, $includedextensions, $includepath, false);
	return $files[mt_rand(0, sizeof($files)-1)];
}
/**
 * @return int
 * @param $dir string
 * @param $extensions array
 * @param $including bool
 * @desc Gets the number of dirs existing in a given dir, excluding (default) or including only the ones wich extension is in $extensions
 */
function countDirs($dir,$extensions=array(),$including=false)
{
	$currentdir = getcwd();
	chdir($dir);
	$handle = opendir(".");
	$count = 0;
	while ($file = readdir($handle))
	{
		if($including)
		{
			if(is_dir($file) && $file!="." && $file!=".." && in_array(getExtension($file),$extensions))
				$count++;
		}
		else
		{
			if(is_dir($file) && $file!="." && $file!=".." && !in_array(getExtension($file),$extensions))
				$count++;
		}
	}
	chdir($currentdir);
	return $count;
}
/**
 * @return int
 * @param $dir string
 * @param $extensions array
 * @param $including bool
 * @desc Gets the number of files existing in a given dir, excluding (default) or including only the ones wich extension is in $extensions
 */
function countFiles($dir,$extensions=array(),$including=false)
{
	$currentdir = getcwd();
	chdir($dir);
	$handle = opendir(".");
	$count = 0;
	while ($file = readdir($handle))
	{
		if($including)
		{
			if(is_file($file) && in_array(getExtension($file),$extensions))
				$count++;
		}
		else
		{
			if(is_file($file) && !in_array(getExtension($file),$extensions))
				$count++;
		}
	}
	chdir($currentdir);
	return $count;
}
/**
 * @return string
 * @param $file string
 * @param $tolower bool
 * @desc Gets the extension of a given file, in lowercase if $tolower
 */
function getExtension($file, $tolower=true)
{
	$file = basename($file);
	$pos = strrpos($file, '.');

	if($file == '' || $pos === false)
	{
		return '';
	}

	$extension = substr($file, $pos+1);
	if($tolower)
	{
		$extension = strtolower($extension);
	}

	return $extension;
}
/**
 * @return string
 * @param $file string
 * @desc Gets the name of a given file
 */
function getName($file, $tolower=false)
{
	$name = ($dotpos = strrpos($file, '.')) ? substr($file, 0, $dotpos) : $file;
	return $tolower ? mb_strtolower($name) : $name;
}
/**
 * @return bool
 * @param $dir string
 * @param $deleteevenifnotempty bool
 * @desc Deletes a dir.
 */
function deleteDir($dir, $deleteevenifnotempty=true)
{
	if(is_dir($dir))
	{
		if($deleteevenifnotempty || is_emptyDir($dir))
		{
			drainDir($dir);
			return rmdir($dir);
		}

		return false;
	}

	return true;
}
/**
 * @return bool
 * @param $dir string
 * @param $createifnotexists bool
 * @param $mode int
 * @desc Drain a dir.
 */
function drainDir($dir, $createifnotexists=true, $mode=0755)
{
	if(is_dir($dir))
	{
		$currentdir = getcwd();
		chdir($dir);
		$handle = opendir(".");
		while($file = readdir($handle))
		{
			if(is_dir($file) && $file!="." && $file!="..")
				deleteDir($file);
			elseif(is_file($file))
				unlink($file);
		}
		closedir($handle);
		chdir($currentdir);
	}
	elseif($createifnotexists)
	{
		mkdir($dir, $mode);
	}
	return is_emptyDir($dir);
}
/**
 * @return bool
 * @param $dir string
 * @param $mode int
 * @param $drainifexists
 * @desc Makes a dir.
 */
function makeDir($dir, $mode=0755, $drainifexists=false)
{
	if(is_dir($dir))
	{
		if($drainifexists)
		{
			drainDir($dir);
		}
	}
	else
	{
		$padre = dirname($dir);
		if($padre && makeDir($padre))
		{
			mkdir($dir, $mode);
		}
	}

	return is_dir($dir);
}
/**
 * @return bool
 * @param string
 * @desc Checks if a given path is a dir and is empty.
 */
function is_emptyDir($dir)
{
	if(is_dir($dir))
	{
		$isempty = true;
		$currentdir = getcwd();
		chdir($dir);
		$handle = opendir(".");
		while($file = readdir($handle))
		{
			if((is_dir($file) && $file!="." && $file!="..") || is_file($file))
			{
				$isempty = false;
				break;
			}
		}
		closedir($handle);
		chdir($currentdir);
		return $isempty;
	}
	return false;
}

function dirsize($dir)
{
	$size = 0;
	$pos = strrpos($dir,'/');
	if($pos == (strlen($dir) - 1))
	{
		$dir = substr($dir, 0, $pos);
	}
	$currentdir = getcwd();
	if(chdir($dir))
	{
		$handle = opendir(".");
		while($file = readdir($handle))
		{
			if(is_dir($file) && $file!="." && $file!="..")
			{
				$size += dirsize($file);
			}
			elseif(is_file($file))
			{
				$size += filesize($file);
			}
		}
		chdir($currentdir);
	}
	return $size;
}
/**
 * Elimina los archivos de un directorio. Devuelve el número de archivos eliminados.
 *
 * @param string $dir
 * @param mixed $exts
 * @param int $antiguedad
 * @return int
 */
function deleteFiles($dir, $exts=null, $antiguedad=0)
{
	$local = opendir($dir);
	$c = 0;
	while ($file = readdir($local))
	{
		if(is_array($exts))
		{
			if(in_array(getExtension($file), $exts))
			{
				if(is_file($dir.$file) && ($antiguedad == 0 || time() - filemtime($dir.$file) > $antiguedad))
				{
					if(@unlink($dir.$file))
					{
						$c++;
					}
				}
			}
		}
		elseif(is_null($exts) || getExtension($file) == $exts)
		{
			if(is_file($dir.$file) && ($antiguedad == 0 || time() - filemtime($dir.$file) > $antiguedad))
			{
				if(@unlink($dir.$file))
				{
					$c++;
				}
			}
		}
	}
	return $c;
}
/**
 * Copia el contenido de un directorio a otro
 *
 * @param string $src
 * @param string $dst
 * @return bool
 */
function copyDir($src, $dst)
{
	$ok = true;
	$dir = opendir($src);
	makeDir($dst);
	while(($file = readdir($dir)) !== false)
	{
		if($file != '.' && $file != '..')
		{
			if(is_dir($src.'/'.$file))
			{
				$ok &= copyDir($src.'/'.$file, $dst.'/'.$file);
			}
			else
			{
				$ok &= copy($src.'/'.$file, $dst.'/'.$file);
			}
		}
	}
	closedir($dir);

	return $ok;
}
/**
 * @desc Comprime un archivo
 * @param string $nom_arxiu
 * @return string
 * PHP5
function comprimir($filename)
{
	try
	{
		$fptr = fopen($filename, "rb");
		$dump = fread($fptr, filesize($filename));
		fclose($fptr);

		//Comprime al máximo nivel, 9
		$gzbackupData = gzencode($dump,9);

		$fptr = fopen($filename.".gz", "wb");
		fwrite($fptr, $gzbackupData);
		fclose($fptr);

		//Devuelve el nombre del archivo comprimido
		return $filename.".gz";
	}
	catch(Exception $ex)
	{
		return false;
	}
}*/
