<?php
/**
 * Common use functions - numbers
 * @author José Carlos Cruz Parra AKA internia
 * josecarlos@programadorphpfreelance.com
 * http://programadorphpfreelance.com
 * 2008-2012
 * This code is released under the GNU General Public License.
 */

 function medidaStr2Num($str, $unidad=1)
{
	$num = '';
	$med = '';
	for($c=0,$len=mb_strlen($str); $c<$len; $c++)
	{
		$char = mb_substr($str, $c, 1);
		if(mb_stripos('1234567890.', $char) !== false)
		{
			$num .= $char;
		}
		elseif(mb_stripos('abcdefghijklmnopqrstuvwxyz', $char) !== false)
		{
			$med .= $char;
		}
	}

	switch(mb_strtolower($med))
	{
		case 'mm':
		case 'gr':
		case 'g':
			$num = ($num * $unidad) / 1000;
			break;
		case 'cm':
		case 'cg':
			$num = ($num * $unidad) / 100;
			break;
		case 'dm':
		case 'dg':
			$num = ($num * $unidad) / 10;
			break;
		case 'm':
		case 'kg':
			$num = $num * $unidad;
			break;
		case 'hm':
			$num = $num * $unidad * 100;
			break;
		case 'km':
		case 't':
			$num = $num * $unidad * 1000;
			break;
	}

	return $num;
}
/**
 * Formats a number as currency
 *
 * @param double $amount
 * @param bool $showcurrsymbol
 * @param bool $uselocalcurrencysymbol
 * @return string
 */
function currency($amount, $showcurrsymbol=true, $uselocalcurrencysymbol=true)
{
    $locale_info = localeconv();
	$number = number_format($amount,2,$locale_info["mon_decimal_point"],$locale_info["mon_thousands_sep"]);
	$curr_symbol = $uselocalcurrencysymbol ? $locale_info["currency_symbol"] : $locale_info["int_curr_symbol"];
	if($amount < 0)
	{
		//TODO: negative values,...
		if($showcurrsymbol)
		{
			if($locale_info["p_cs_precedes"])
			{
				if($locale_info["p_sep_by_space"])
					$curr_symbol .= " ";
				return $curr_symbol.$number;
			}
			else
			{
				if($locale_info["p_sep_by_space"])
					$curr_symbol = " ".$curr_symbol;
				return $number.$curr_symbol;
			}
		}
		return $number;
	}
	else
	{
		if($showcurrsymbol)
		{
			if($locale_info["p_cs_precedes"])
			{
				if($locale_info["p_sep_by_space"])
					$curr_symbol .= " ";
				return $curr_symbol.$number;
			}
			else
			{
				if($locale_info["p_sep_by_space"])
					$curr_symbol = " ".$curr_symbol;
				return $number.$curr_symbol;
			}
		}
		return $number;
	}
}
/**
 * Aplica un porcentaje a una cantidad dada
 *
 * @param double $importe
 * @param float $porcentaje
 * @param int $base
 * @return double
 */
function aplicarPorcentaje($cantidad, $porcentaje, $base=100)
{
	return $cantidad + (($cantidad / $base) * $porcentaje);
}
/**
 * Prepara un número para guardarlo en la base de datos.
 * No valida el número ni comprueba si es válido.
 *
 * @param mixed $n
 * @return mixed
 */
function prepararNumero($n)
{
	if(strpos($n, ',') !== false)
	{
		$n = str_replace(array('.',','), array('','.'), $n);
	}

	return $n;
}
?>