<?php
/**
 * Jose Carlos Cruz Parra
 * programadorphpfreelance.com
 * 2012-12-05
 * para Francisco Ortega
 *
 */
class KPTL_ActuacionesAdmin
{
	const STATUS_INICIO = 0;
	const STATUS_CONTRASENAINCORRECTA = 1;
	const STATUS_ENVIOINCORRECTO = 2;

	private static $_status;

	/**
	 * Realiza una actuación
	 *
	 * @param string $clave
	 * @param array $vars
	 * @param array $params
	 * @param bool $xajax
	 * @return bool
	 */
    public static function Act($clave, $vars=array(), $params=array(), $xajax=false)
    {
    	global $db, $kptl_config;

    	if(empty($kptl_config))
    	{
    		$kptl_config = SWPHP::_GetConfigValues('kapitaltruth', false, 'kptl_config');
    	}

    	KPTL_ActuacionesAdmin::$_status = KPTL_ActuacionesAdmin::STATUS_INICIO;

   		$response = $xajax ? new xajaxResponse() : null;

    	if($kptl_config['minadmin'] > 1
    		&& $db->GetValueQuery("SELECT COUNT(*) FROM ".SWPHP_DBPREFIX."users WHERE activo = '1' AND perfil_id IN ('".implode("','", explode(',', $kptl_config['perfilesadmin']))."')", false) >= $kptl_config['minadmin']
    		&& $db->GetValueQuery("SELECT firma FROM ".SWPHP_DBPREFIX."kptl_actuacionesadmin WHERE clave = '{$clave}'", false) > 0
    	)
    	{
    		if(isset($_SESSION['kapitaltruth']['actuacionadmin']))
    		{
	    		$control = $_SESSION['kapitaltruth']['actuacionadmin']['control'];
		    	$clave = $_SESSION['kapitaltruth']['actuacionadmin']['clave'];
				$vars = $_SESSION['kapitaltruth']['actuacionadmin']['vars'];
				$params = $_SESSION['kapitaltruth']['actuacionadmin']['params'];

				unset($_SESSION['kapitaltruth']['actuacionadmin']);

	    		if(KPTL_ActuacionesAdmin::_CheckFirma($control))
	    		{
	    			return KPTL_ActuacionesAdmin::_Act($clave, $vars, $params, $xajax, $response);
	    		}
    		}

    		if(KPTL_ActuacionesAdmin::$_status != KPTL_ActuacionesAdmin::STATUS_ENVIOINCORRECTO)
    		{
    			$response = KPTL_ActuacionesAdmin::_PedirFirma($clave, $vars, $params, $xajax, $response);
    		}

    		return $xajax ? $response : false;
    	}

    	unset($_SESSION['kapitaltruth']['actuacionadmin']);

    	return KPTL_ActuacionesAdmin::_Act($clave, $vars, $params, $xajax, $response);
    }
	/**
	 * Pide la firma del admin o admins
	 *
	 * @param string $clave
	 * @param array $vars
	 * @param array $params
	 * @param bool $xajax
	 * @param mixed $response
	 * @return mixed
	 */
    private static function _PedirFirma($clave, $vars, $params, $xajax=false, $response=null)
    {
		global $db, $kptl_config, $smarty, $admin_template;

		$control = GenerateRandomKey(12);

		$_SESSION['kapitaltruth']['actuacionadmin'] = array(
			'control'=>$control,
			'clave'=>$clave,
			'vars'=>$vars,
			'params'=>$params,
			);

		$kptl_act = $db->GetRow("SELECT a.clave, al.nombre FROM ".SWPHP_DBPREFIX."kptl_actuacionesadmin AS a JOIN ".SWPHP_DBPREFIX."kptl_actuacionesadmin_idiomas AS al ON al.actuacion_id = a.id AND al.idioma_id = '{$_SESSION['SWPHP_IDIOMA_ACTUAL']}' WHERE clave = '{$clave}'");

		if($xajax)
		{
			$response->assign('pfActNombre', 'innerHTML', $kptl_act['nombre']);
			$response->assign('pfControl', 'value', $control);
			$response->script("popPedirFirma.show(); ajustarVertical('popPedirFirma')");
		}
		else
		{
			$smarty->assign(array(
				'kptl_config'=>$kptl_config,
				'kptl_act'=>$kptl_act,
				'kptl_control'=>$control,
				));

			$admin_template = SWPHP::_GetModuleTemplate('pedirfirma.tpl');
		}

		return $response;
    }
	/**
	 * Comprueba si la firma conjunta es válida
	 *
	 * @param string $control
	 * @param array $data
	 * @return bool
	 */
    private static function _CheckFirma($control, $data=null)
    {
    	global $db, $kptl_config;

    	if(is_null($data))
    	{
    		$data = $_POST;
    	}

    	if(isset($data['pass']) && is_array($data['pass']) && sizeof($data['pass']) == $kptl_config['minadmin'] && isset($data['control']) && $data['control'] == $control)
		{
			$n = 0;
			$arr = array_unique($data['pass']);
			foreach($arr as $pass)
			{
				$n += $db->GetValueQuery("SELECT COUNT(*) FROM ".SWPHP_DBPREFIX."users WHERE pass = '".SWPHP::_Encrypt($pass)."' AND perfil_id IN('".implode("','", explode(',', $kptl_config['perfilesadmin']))."')", false);
			}

			if($kptl_config['minadmin'] == $n)
			{
				return true;
			}

			KPTL_ActuacionesAdmin::$_status = KPTL_ActuacionesAdmin::STATUS_CONTRASENAINCORRECTA;

			SWPHP::_Msg(l('Alguna constraseña no es correcta', 'kapitaltruth'), 'error');
		}
		else
		{
			KPTL_ActuacionesAdmin::$_status = KPTL_ActuacionesAdmin::STATUS_ENVIOINCORRECTO;

			SWPHP::_Msg(l('El envío no se ha realizado correctamente', 'kapitaltruth'), 'error');
		}

		return false;
    }
	/**
	 * De verdad realiza una actuación
	 *
	 * @param string $_clave
	 * @param array $_vars
	 * @param array $_params
	 * @param bool $xajax
	 * @param mixed $response
	 * @return bool
	 */
    private static function _Act($_clave, $_vars, $_params, $xajax=false, $response=null)
    {
    	global $db, $swphp_permiso_id, $swphp_idiomas;

    	foreach($_vars as $_key=>$_value)
    	{
    		$$_key = $_value;
    	}

    	if($xajax)
    	{
	    	switch($_clave)
	    	{
	    		case 'ModUsuario':
                    $user_id = $db->GetValue("user_id", SWPHP_DBPREFIX."kptl_usuarios", "WHERE id = '{$id}'");
	    			$response = SWPHP_Xajax::_Switch($response, array($user_id, $id), SWPHP_DBPREFIX."users");
                    break;

	    		case 'DelUsuario':
			    	switch(kapitaltruth_usuarios_eliminarUsuario($id))
					{
						case -1:
                            $user_id = $db->GetValue("user_id", SWPHP_DBPREFIX."kptl_usuarios", "WHERE id = '{$id}'");
                            $activo = $db->GetValueQuery("SELECT activo FROM ".SWPHP_DBPREFIX."users WHERE id = '{$user_id}'", false);
                            $response->script("gebi('aActivar{$id}').title = '".($activo ? 'Desactivar' : 'Activar')."'");
                            $response->script("gebi('aActivar{$id}').innerHTML = '".SWPHP::_Icon($activo ? 'green_button.png' : 'red_button.png', 16)."'");
                            break;
                        case 0:
                        case false:
                        default:
                            //nada
                            break;
                        case 1:
                        case true:
                            $response->script("gebi('aEliminar{$id}').parentNode.parentNode.style.display = 'none';");
                            break;
					}
					break;

                case 'ModGrupo':
	    			$response = SWPHP_Xajax::_Switch($response, $id, SWPHP_DBPREFIX."kptl_grupos");
	    			break;

	    		case 'DelGrupo':
	    			if(kapitaltruth_grupos_eliminarGrupo($id))
					{
						$response->script("gebi('aEliminar{$id}').parentNode.parentNode.style.display = 'none';");
					}
	    			break;

				case 'ModCategoria':
	    			$response = SWPHP_Xajax::_Switch($response, $id, SWPHP_DBPREFIX."kptl_categorias");
	    			break;

	    		case 'DelCategoria':
	    			if(kapitaltruth_categorias_eliminarCategoria($id))
					{
						$response->script("gebi('aEliminar{$id}').parentNode.parentNode.style.display = 'none';");
					}
	    			break;

	    		case 'ModPregunta':
	    			$response = SWPHP_Xajax::_Switch($response, $id, SWPHP_DBPREFIX."kptl_preguntas");
	    			break;

	    		case 'DelPregunta':
	    			if(kapitaltruth_preguntas_eliminarPregunta($id))
					{
						$response->script("gebi('aEliminar{$id}').parentNode.parentNode.style.display = 'none';");
					}
	    			break;

	    		default:
	    			SWPHP::_Msg(l('Actuación no esperada', 'kapitaltruth'), 'aviso');

	    			break;
	    	}

	    	return $response;
    	}
    	else
    	{
    		switch($_clave)
	    	{
	    		case 'ModConfig':
	    			$conceptos = $db->GetValuesQuery("SELECT DISTINCT concepto FROM ".SWPHP_DBPREFIX."{$tabla} WHERE concepto LIKE '{$q}%'", false);
					foreach($conceptos as $concepto)
					{
						if(!isset($_params[$concepto]))
						{
							$_params[$concepto] = array(); //Tengo que hacer esto para los casos en que el concepto sólo tiene un parámetro y es un check
						}
					}

					foreach($_params as $key=>$data)
					{
						if($db->Exists($key, "concepto", SWPHP_DBPREFIX.$tabla))
						{
							if($key == 'kapitaltruth' && isset($data['habilitarkptl']) && $data['habilitarkptl'])
							{
								if(!kapitaltruth_configuracion_sePuedeHabilitarKPTL())
								{
									unset($data['habilitarkptl']);
								}
							}

							SWPHP::_SetConfigValues(kapitaltruth_configuracion_prepararDatos($data, $key), $key, true, $tabla, $swphp_permiso_id < SWPHP_PERMISO_ACCESOTOTAL);
						}
					}
					SWPHP::_Msg(l('Se ha guardado la configuración', 'kapitaltruth'), 'check');
					//SWPHP::_Redirigir($swphp_config['url'].SWPHP_ADMIN_DIR);
					break;

	    		case 'AddUsuario':
	    			if(kapitaltruth_usuarios_validarDatos($_params))
					{
						$convertir = isset($_params['convertir']) && $_params['convertir'] ? intval($_params['convertir']) : false;

						if($convertir && key_exists($convertir, kapitaltruth_usuarios_getConvertibles()))
						{
							$user_id = $convertir;
						}
						else
						{
							if(SWPHP::_ExecuteQuery(
								buildQuery_Insert(kapitaltruth_usuarios_prepararDatosUser($_params), SWPHP_DBPREFIX."users"),
								"",
								l("No se ha podido insertar el registro", 'kapitaltruth'))
								)
							{
								$user_id = $db->Insert_id();
							}
						}

						if(isset($user_id))
						{
							$datos = kapitaltruth_usuarios_prepararDatosUsuario($_params);
							$datos["user_id"] = $user_id;

							SWPHP::_ExecuteQuery(
								buildQuery_Insert($datos, SWPHP_DBPREFIX."kptl_usuarios"),
								l("Usuario creado correctamente", 'kapitaltruth'),
								l("No se ha podido insertar el registro", 'kapitaltruth')
								);
						}
					}
	    			break;

	    		case 'ModUsuario':
	    			if(kapitaltruth_usuarios_validarDatos($_params, $id))
					{
						if(SWPHP::_ExecuteQuery(
							buildQuery_Update(kapitaltruth_usuarios_prepararDatosUser($_params, $user_id), SWPHP_DBPREFIX."users", $user_id),
							"",
							l("No se ha podido actualizar el registro", 'kapitaltruth'))
							)
						{
							SWPHP::_ExecuteQuery(
								buildQuery_Update(kapitaltruth_usuarios_prepararDatosUsuario($_params, $id), SWPHP_DBPREFIX."kptl_usuarios", $id),
								l("Usuario actualizado correctamente", 'kapitaltruth'),
								l("No se ha podido actualizar el registro", 'kapitaltruth')
								);
						}
					}
	    			break;

                case 'AddGrupo':
	    			if(kapitaltruth_grupos_validarDatos($_params))
					{
						if(SWPHP::_ExecuteQuery(
							buildQuery_Insert(kapitaltruth_grupos_prepararDatos($_params), SWPHP_DBPREFIX."kptl_grupos"),
							l("Grupo registrado correctamente", 'kapitaltruth'),
							l("No se ha podido insertar el registro", 'kapitaltruth')
							))
						{
							$id = $db->Insert_Id();
							foreach($swphp_idiomas as $idioma)
							{
								SWPHP::_ExecuteQuery(
									buildQuery_Insert(kapitaltruth_grupos_prepararDatosIdioma($_params['idiomas'], $idioma['id'], $id), SWPHP_DBPREFIX."kptl_grupos_idiomas"),
									'',
									l("No se ha podido guardar la información en", 'kapitaltruth')." {$idioma['nombre']}"
								);
							}
						}
					}
					break;

	    		case 'ModGrupo':
	    			if(kapitaltruth_grupos_validarDatos($_params))
					{
						if(SWPHP::_ExecuteQuery(
							buildQuery_Update(kapitaltruth_grupos_prepararDatos($_params, $id), SWPHP_DBPREFIX."kptl_grupos", $id),
							l("Grupo actualizado correctamente", 'kapitaltruth'),
							l("No se ha podido actualizar el registro", 'kapitaltruth')
							))
						{
							foreach($swphp_idiomas as $idioma)
							{
								$query = $db->Exists($idioma['id'], 'idioma_id', SWPHP_DBPREFIX.'kptl_grupos_idiomas', "WHERE grupo_id = '{$id}'") ?
									buildQuery_Update(kapitaltruth_grupos_prepararDatosIdioma($_params['idiomas'], $idioma['id'], $id), SWPHP_DBPREFIX."kptl_grupos_idiomas", array('grupo_id'=>$id, 'idioma_id'=>$idioma['id']))
									:
									buildQuery_Insert(kapitaltruth_grupos_prepararDatosIdioma($_params['idiomas'], $idioma['id'], $id), SWPHP_DBPREFIX."kptl_grupos_idiomas");

								SWPHP::_ExecuteQuery(
									$query,
									'',
									l("No se ha podido guardar la información en", 'kapitaltruth')." {$idioma['nombre']}"
								);
							}
						}
					}
					break;

	    		case 'AddCategoria':
	    			if(kapitaltruth_categorias_validarDatos($_params))
					{
						if(SWPHP::_ExecuteQuery(
							buildQuery_Insert(kapitaltruth_categorias_prepararDatos($_params), SWPHP_DBPREFIX."kptl_categorias"),
							l("Categoría registrada correctamente", 'kapitaltruth'),
							l("No se ha podido insertar el registro", 'kapitaltruth')
							))
						{
							$id = $db->Insert_Id();
							foreach($swphp_idiomas as $idioma)
							{
								SWPHP::_ExecuteQuery(
									buildQuery_Insert(kapitaltruth_categorias_prepararDatosIdioma($_params['idiomas'], $idioma['id'], $id), SWPHP_DBPREFIX."kptl_categorias_idiomas"),
									'',
									l("No se ha podido guardar la información en", 'kapitaltruth')." {$idioma['nombre']}"
								);
							}
						}
					}
					break;

	    		case 'ModCategoria':
	    			if(kapitaltruth_categorias_validarDatos($_params))
					{
						if(SWPHP::_ExecuteQuery(
							buildQuery_Update(kapitaltruth_categorias_prepararDatos($_params, $id), SWPHP_DBPREFIX."kptl_categorias", $id),
							l("Categoría actualizada correctamente", 'kapitaltruth'),
							l("No se ha podido actualizar el registro", 'kapitaltruth')
							))
						{
							foreach($swphp_idiomas as $idioma)
							{
								$query = $db->Exists($idioma['id'], 'idioma_id', SWPHP_DBPREFIX.'kptl_categorias_idiomas', "WHERE categoria_id = '{$id}'") ?
									buildQuery_Update(kapitaltruth_categorias_prepararDatosIdioma($_params['idiomas'], $idioma['id'], $id), SWPHP_DBPREFIX."kptl_categorias_idiomas", array('categoria_id'=>$id, 'idioma_id'=>$idioma['id']))
									:
									buildQuery_Insert(kapitaltruth_categorias_prepararDatosIdioma($_params['idiomas'], $idioma['id'], $id), SWPHP_DBPREFIX."kptl_categorias_idiomas");

								SWPHP::_ExecuteQuery(
									$query,
									'',
									l("No se ha podido guardar la información en", 'kapitaltruth')." {$idioma['nombre']}"
								);
							}
						}
					}
					break;

	    		case 'AddPregunta':
	    			if(kapitaltruth_preguntas_validarDatos($_params))
					{
						if(SWPHP::_ExecuteQuery(
							buildQuery_Insert(kapitaltruth_preguntas_prepararDatos($_params), SWPHP_DBPREFIX."kptl_preguntas"),
							l("Pregunta registrada correctamente", 'kapitaltruth'),
							l("No se ha podido insertar el registro", 'kapitaltruth')
							))
						{
							$id = $db->Insert_Id();
							foreach($swphp_idiomas as $idioma)
							{
								SWPHP::_ExecuteQuery(
									buildQuery_Insert(kapitaltruth_preguntas_prepararDatosIdioma($_params['idiomas'], $idioma['id'], $id), SWPHP_DBPREFIX."kptl_preguntas_idiomas"),
									'',
									l("No se ha podido guardar la información en", 'kapitaltruth')." {$idioma['nombre']}"
								);
							}
						}
					}
					break;

	    		case 'ModPregunta':
	    			if(kapitaltruth_preguntas_validarDatos($_params))
					{
						if(SWPHP::_ExecuteQuery(
							buildQuery_Update(kapitaltruth_preguntas_prepararDatos($_params, $id), SWPHP_DBPREFIX."kptl_preguntas", $id),
							l("Pregunta actualizada correctamente", 'kapitaltruth'),
							l("No se ha podido actualizar el registro", 'kapitaltruth')
							))
						{
							foreach($swphp_idiomas as $idioma)
							{
								$query = $db->Exists($idioma['id'], 'idioma_id', SWPHP_DBPREFIX.'kptl_preguntas_idiomas', "WHERE pregunta_id = '{$id}'") ?
									buildQuery_Update(kapitaltruth_preguntas_prepararDatosIdioma($_params['idiomas'], $idioma['id'], $id), SWPHP_DBPREFIX."kptl_preguntas_idiomas", array('pregunta_id'=>$id, 'idioma_id'=>$idioma['id']))
									:
									buildQuery_Insert(kapitaltruth_preguntas_prepararDatosIdioma($_params['idiomas'], $idioma['id'], $id), SWPHP_DBPREFIX."kptl_preguntas_idiomas");

								SWPHP::_ExecuteQuery(
									$query,
									'',
									l("No se ha podido guardar la información en", 'kapitaltruth')." {$idioma['nombre']}"
								);
							}
						}
					}
					break;

	    		default:
	    			SWPHP::_Msg(l('Actuación no esperada', 'kapitaltruth'), 'aviso');

	    			return false;
	    	}
    	}

    	return true;
    }
}
?>