<?php
/**
 * Jose Carlos Cruz Parra
 * programadorphpfreelance.com
 * 2012-04-08
 * para Francisco Ortega
 *
 */
class KPTL_User extends SWPHP_Reg
{
    /**
     * @var SWPHP_User
     */
	private $_user;

    /**
	 * Constructor
	 */
	function __construct($kptl_user_id=null)
    {
        $this->_camposreadonly[] = 'edad';
        $this->_camposserializados[] = 'grupos';

        parent::__construct(SWPHP_DBPREFIX.'kptl_usuarios', $kptl_user_id);

        $this->_data['edad'] = ($this->_data['fechanacimiento'] == '0000-00-00' || !$this->_data['fechanacimiento']) ? false : calcularEdadStr($this->_data['fechanacimiento']);
    }
    /**
	 * Asigna el identificador de la conexión a la base de datos
	 *
     * @param resource
     */
    public function SetLinkIdentifier($dbcon)
    {
    	parent::SetLinkIdentifier($dbcon);

    	$this->_user->SetLinkIdentifier($dbcon);
    }
    /**
     * Carga un usuario de kptl
     *
     * @param int $kptl_user_id
     * @return bool
     */
    public function Load($kptl_user_id=null)
    {
		if(parent::Load($kptl_user_id))
		{
			return $this->_user->Load($this->_data['user_id']);
		}

		return false;
    }
    /**
	 * Resetea los datos (campos)
	 */
	public function ResetData()
    {
    	parent::ResetData();

    	$this->_data['edad'] = false;

    	$this->_user = new SWPHP_User();
    }
    /**
	 * Establece el valor de un campo
	 *
	 * @param string $campo
	 * @param mixed $valor
	 * @return bool
	 */
	public function SetValue($campo, $valor)
    {
    	return parent::SetValue($campo, $valor) || $this->_user->SetValue($campo, $valor);
    }
    /**
	 * Obtiene el valor de un campo (null si no lo encuentra)
	 *
	 * @param string $campo
	 * @param bool $htmlentities
	 * @return mixed
	 */
	public function GetValue($campo, $htmlentities=true)
	{
		$valor = parent::GetValue($campo, $htmlentities);

		return is_null($valor) ? $this->_user->GetValue($campo, $htmlentities) : $valor;
	}
	/**
     * Obtiene el id de la autovaloración correspondiente a la fecha dada,
     * y si ésta es null, a la fecha actual
     *
     * @param string $fecha
     * @param bool $crearSiNoExiste
     * @return int
     */
    public function GetAutoValoracionId($fecha=null, $crearSiNoExiste=false)
    {
    	if(is_null($fecha))
    	{
    		$fecha = date('Y-m-d');
    	}

    	if($crearSiNoExiste)
    	{
    		$autovaloracion_id = $this->_db->GetValueQuery("SELECT id FROM ".SWPHP_DBPREFIX."kptl_autovaloraciones WHERE user_id = '".$this->id()."' AND fecha = '{$fecha}'", false);

    		if(!$autovaloracion_id)
    		{
    			if($this->_db->Execute(sprintf("INSERT INTO ".SWPHP_DBPREFIX."kptl_autovaloraciones(id, fecha, user_id) VALUES(null, '%s', %u)", $fecha, $this->id())))
				{
    				$autovaloracion_id = $this->_db->Insert_id();
    			}
    			//else $autovaloracion_id = null;
    		}
    	}
		else
		{
			$autovaloracion_id = $this->_db->GetValueQuery("SELECT id FROM ".SWPHP_DBPREFIX."kptl_autovaloraciones WHERE user_id = '".$this->id()."' AND fecha <= '{$fecha}' ORDER BY fecha DESC", false);
		}

    	return $autovaloracion_id;
    }
    /**
     * Obtiene un array asociativo con las respuestas a las preguntas en auto-valoración,
     * donde los índices son los id de pregunta y los valores los id de respuesta.
     *
     * @param string $fecha
     * @return array
     */
    public function GetAutoValoracion($fecha=null)
    {
    	$autovaloracion_id = $this->GetAutoValoracionId($fecha);

    	return $autovaloracion_id ?
			$this->_db->GetArrayForHTMLSelect('pregunta_id', 'respuesta_id', SWPHP_DBPREFIX.'kptl_autovaloraciones_preguntas', "WHERE autovaloracion_id = '{$autovaloracion_id}'", null, true, false)
			:
			$this->_db->GetArrayForHTMLSelectQuery("SELECT id, -1 FROM ".SWPHP_DBPREFIX."kptl_preguntas", null, true, false);
    }
    /**
     * Obtiene el id de la valoración sobre otro usuario correspondiente a la fecha dada,
     * y si ésta es null, a la fecha actual
     *
     * @param int $user_idpasivo
     * @param string $fecha
     * @param bool $crearSiNoExiste
     * @return int
     */
    public function GetValoracionId($user_idpasivo, $fecha=null, $crearSiNoExiste=false)
    {
    	if(is_null($fecha))
    	{
    		$fecha = date('Y-m-d');
    	}

    	if($crearSiNoExiste)
    	{
    		$valoracion_id = $this->_db->GetValueQuery("SELECT id FROM ".SWPHP_DBPREFIX."kptl_valoraciones WHERE user_idactivo = '".$this->id()."' AND user_idpasivo = '{$user_idpasivo}' AND fecha = '{$fecha}'", false);

    		if(!$valoracion_id)
    		{
    			if($this->_db->Execute(sprintf("INSERT INTO ".SWPHP_DBPREFIX."kptl_valoraciones(id, fecha, user_idactivo, user_idpasivo, puntos) VALUES(null, '%s', %u, %u, 0)", $fecha, $this->id(), $user_idpasivo)))
    			{
    				$valoracion_id = $this->_db->Insert_id();
    			}
    			//else $valoracion_id = null;
    		}
    	}
		else
		{
			$valoracion_id = $this->_db->GetValueQuery("SELECT id FROM ".SWPHP_DBPREFIX."kptl_valoraciones WHERE user_idactivo = '".$this->id()."' AND user_idpasivo = '{$user_idpasivo}' AND fecha <= '{$fecha}' ORDER BY fecha DESC", false);
		}

    	return $valoracion_id;
    }
    /**
     * Obtiene un array asociativo con las respuestas a las preguntas en valoración de otro usuario,
     * donde los índices son los id de pregunta y los valores los id de respuesta.
     *
     * @param int $user_idpasivo
     * @param string $fecha
     * @return array
     */
    public function GetValoracion($user_idpasivo, $fecha=null)
    {
    	$valoracion_id = $this->GetValoracionId($user_idpasivo, $fecha);

    	return $valoracion_id ?
			$this->_db->GetArrayForHTMLSelect('pregunta_id', 'respuesta_id', SWPHP_DBPREFIX.'kptl_valoraciones_preguntas', "WHERE valoracion_id = '{$valoracion_id}'", null, true, false)
			:
			$this->_db->GetArrayForHTMLSelectQuery("SELECT id, -1 FROM ".SWPHP_DBPREFIX."kptl_preguntas", null, true, false);
    }
    /**
     * Obtiene el número de preguntas actuales que el usuario aún no ha respondido
     *
     * @return int
     */
    public function GetNumPreguntasPendientes()
	{
		$autovaloracion_id = $this->GetAutoValoracionId();

		return $this->_db->GetValueQuery("SELECT COUNT(*) FROM ".SWPHP_DBPREFIX."kptl_preguntas WHERE id NOT IN(SELECT pregunta_id FROM ".SWPHP_DBPREFIX."kptl_autovaloraciones_preguntas WHERE autovaloracion_id = '{$autovaloracion_id}')", false);
	}
	/**
	 * Obtiene una frase con información acerca del número de preguntas actuales que el usuario aún no ha respondido
	 *
	 * @return string
	 */
	public function GetInfoPreguntasPendientes()
	{
		$preguntasPendientes = $this->GetNumPreguntasPendientes();

		return $preguntasPendientes > 0 ?
			"<span class=\"alerta\"><b>{$preguntasPendientes}</b> ".l('preguntas de auto-valoración están pendientes de respuesta.', 'kapitaltruth')."</span>"
			:
			l('Ninguna pregunta de auto-valoración está pendiente de respuesta.', 'kapitaltruth');
	}
	/**
	 * Obtiene el id de la respuesta que el usuario dio en autovaloración para la pregunta $pregunta_id
	 *
	 * @param int $pregunta_id
	 * @param int $autovaloracion_id
	 * @param string $fecha
	 * @return int
	 */
	public function GetRespuestaId($pregunta_id, $autovaloracion_id=null, $fecha=null)
	{
		if(is_null($autovaloracion_id))
		{
			$autovaloracion_id = $this->GetAutoValoracionId($fecha);
		}

		return $this->_db->GetValueQuery("SELECT respuesta_id FROM ".SWPHP_DBPREFIX."kptl_autovaloraciones_preguntas WHERE autovaloracion_id = '{$autovaloracion_id}' AND pregunta_id = '{$pregunta_id}'", false);
	}
	/**
	 * Obtiene los usuarios del conjunto todos (todos los usuarios excepto yo)
	 *
	 * @param bool $count
	 * @param string $filtro
	 * @return mixed
	 */
	public function GetConjuntoTodos($count=false, $filtro='')
	{
		return $this->GetConjunto('todos', $count, $filtro);
	}
	/**
	 * Obtiene los usuarios del conjunto nunca (que no ha valorado nunca)
	 *
	 * @param bool $count
	 * @param string $filtro
	 * @return mixed
	 */
	public function GetConjuntoNunca($count=false, $filtro='')
	{
		return $this->GetConjunto('nunca', $count, $filtro);
	}
	/**
	 * Obtiene los usuarios del conjunto antes (que ha valorado ya antes)
	 *
	 * @param bool $count
	 * @param string $filtro
	 * @return mixed
	 */
	public function GetConjuntoAntes($count=false, $filtro='')
	{
		return $this->GetConjunto('antes', $count, $filtro);
	}
	/**
	 * Obtiene los usuarios del conjunto indicado
	 *
	 * @param string $conjunto
	 * @param bool $count
	 * @param string $filtro
	 * @return mixed
	 */
	public function GetConjunto($conjunto='todos', $count=false, $filtro='')
	{
		return $count ? $this->_db->GetValueQuery($this->Conjunto2Query($conjunto, true, $filtro), false) : $this->_db->GetRows($this->Conjunto2Query($conjunto, false, $filtro));
	}
	/**
	 * Obtiene la consulta SQL correspondiente a un conjunto
	 *
	 * @param string $conjunto
	 * @param bool $count
	 * @param string $filtro
	 * @return string
	 */
	public function Conjunto2Query($conjunto='todos', $count=false, $filtro='')
	{
		switch(strtolower($conjunto))
		{
			default:
			case 'todos':
			case 't':
			case '0':
				$where = "WHERE ku.id != '".$this->id()."'";
				break;
			case 'nunca':
			case 'n':
			case '1':
				$where = "WHERE ku.id != '".$this->id()."' AND ku.id NOT IN(SELECT DISTINCT v.user_idpasivo FROM ".SWPHP_DBPREFIX."kptl_valoraciones AS v WHERE v.user_idactivo = '".$this->id()."')";
				break;
			case 'antes':
			case 'a':
			case '2':
				$where = "WHERE ku.id != '".$this->id()."' AND ku.id IN(SELECT DISTINCT v.user_idpasivo FROM ".SWPHP_DBPREFIX."kptl_valoraciones AS v WHERE v.user_idactivo = '".$this->id()."')";
				break;
		}

		return self::GetQuery(null, $count ? 'COUNT(ku.id)' : null, mergeWhereFiltro($where, $filtro).' AND u.activo = 1'.($count ? '' : ' ORDER BY u.nick'));
	}
	/**
	 * Obtiene la consulta sql para coger usuarios
	 *
	 * @param mixed $ids
	 * @param string $campos
	 * @param string $filtro
	 * @return string
	 */
    public static function GetQuery($ids=null, $campos=null, $filtro='')
    {
    	if(is_null($campos))
    	{
    		$campos = "ku.id, u.nick, u.pass, ku.nombre, u.email, ku.fechanacimiento, ku.puntos_show, ku.puntos_real, ku.puntos_recibidos, ku.puntos_emitidos, ku.fechayhoraalta";
    	}

    	if(is_null($ids))
    	{
    		$aux = '';
    	}
    	elseif(is_numeric($ids) && $ids > 0)
    	{
    		$aux = 'WHERE ku.id = '.intval($ids);
    	}
    	elseif(is_array($ids))
    	{
    		$aux = 'WHERE';
    		$sep = ' ';
    		foreach($ids as $key=>$value)
    		{
				$aux .= sprintf("%s %s = '%s'", $sep, $key, addcslashes($value, "'"));
				$sep = ' AND';
    		}
    	}

    	$filtro = trim($filtro);
    	if($filtro)
    	{
    		if($aux)
    		{
    			if(strtoupper(substr($filtro, 0, 6)) == 'WHERE ')
    			{
    				$filtro = $aux.' AND ('.substr($filtro, 6).')';
    			}
    			else
    			{
    				$filtro = $aux.' '.$filtro;
    			}
    		}
    	}
    	else
    	{
    		$filtro = $aux;
    	}

    	return "SELECT {$campos} FROM ".SWPHP_DBPREFIX."users AS u JOIN ".SWPHP_DBPREFIX."kptl_usuarios AS ku ON ku.user_id = u.id {$filtro}";
    }
    /**
	 * Obtiene un conjunto de usuarios
	 *
	 * @param bool $comoObjeto
	 * @param string $campos
	 * @param string $filtro
	 * @param string $filtroBase
	 * @return array
	 */
    public static function GetUsers($comoObjeto=false, $campos=null, $filtro='ORDER BY u.nick ASC', $filtroBase='WHERE u.activo = 1')
    {
        global $db;

    	$rs = DB_ResultSet::Factory($db->GetExtension());
    	$rs->Set = $db->Execute(self::GetQuery(null, $campos, mergeWhereFiltro($filtroBase, $filtro)));
       	$rows = array();
       	if($comoObjeto)
    	{
    		while($row = $rs->FetchAssoc())
		   	{
		   		$rows[] = new KPTL_User($row['id']);
		   	}
    	}
    	elseif(isset($row['fechanacimiento']))
    	{
			while($row = $rs->FetchAssoc())
		   	{
		   		$row['edad'] = $row['fechanacimiento'] == '0000-00-00' ? false : calcularEdadStr($row['fechanacimiento']);
		   		$rows[] = $row;
		   	}
    	}
    	else
    	{
			while($row = $rs->FetchAssoc())
		   	{
		   		$rows[] = $row;
		   	}
    	}

        return $rows;
    }
    /**
     * Busca usuarios
     *
     * @param string $q
     * @param bool $excluirActual
     * @param bool $soloActivos
     * @return array
     */
    public static function Buscar($q, $excluirActual=true, $soloActivos=true)
    {
    	$filtro = sprintf("AND CONCAT_WS(' ', u.nick, ku.nombre) LIKE _utf8 '%%%s%%' COLLATE utf8_general_ci", addcslashes($q, "'"));
    	if($excluirActual)
    	{
    		global $kptl_user_id;

    		$filtro .= " AND ku.id != '{$kptl_user_id}'";
    	}
    	$filtro .= " ORDER BY u.nick ASC";

    	return self::GetUsers(false, 'ku.id, u.nick, ku.nombre', $filtro, $soloActivos ? 'WHERE u.activo = 1' : 'WHERE 1');
    }
    /**
     * Obtiene una cadena con el (nick) nombre de un usuario
     *
     * @param int $kptl_user_id
     * @param bool $htmlentities
     * @return string
     */
    public static function Id2NickNombre($kptl_user_id, $htmlentities=true)
    {
    	global $db;

    	return $db->GetValueQuery("SELECT CONCAT('(',u.nick,')',' ',ku.nombre) FROM ".SWPHP_DBPREFIX."kptl_usuarios AS ku JOIN ".SWPHP_DBPREFIX."users AS u ON u.id = ku.user_id WHERE ku.id = '{$kptl_user_id}'", $htmlentities);
    }
    /**
	 * Convierte un array de datos de usuario en un objeto KPTL_User
	 *
	 * @param array $row
	 * @return KPTL_User
	 */
    private static function _row2obj($row)
    {
    	if(!isset($row['edad']))
    	{
    		$row['edad'] = $row['fechanacimiento'] == '0000-00-00' ? false : calcularEdadStr($row['fechanacimiento']);
    	}

    	return new KPTL_User($row['id'], $row['nick'], $row['pass'], $row['nombre'], $row['email'], $row['edad'], $row['puntos_show'], $row['puntos_real'], $row['puntos_recibidos'], $row['puntos_emitidos'], $row['fechayhoraalta']);
    }
	/**
	 * Obtiene un conjunto de usuarios
	 *
	 * @param bool $comoObjeto
	 * @param string $campos
	 * @return array
	 */
    public static function query_all($comoObjeto=true, $campos=null, $filtro='')
    {
        global $db;

    	$rs = DB_ResultSet::Factory($db->GetExtension());
    	$rs->Set = $db->Execute(self::GetQuery(null, $campos, $filtro));
       	$rows = array();
       	if($comoObjeto)
    	{
    		while($row = $rs->FetchAssoc())
		   	{
		   		$rows[] = self::_row2obj($row);
		   	}
    	}
    	else
    	{
			while($row = $rs->FetchAssoc())
		   	{
		   		$row['edad'] = $row['fechanacimiento'] == '0000-00-00' ? false : calcularEdadStr($row['fechanacimiento']);
		   		$rows[] = $row;
		   	}
    	}

        return $rows;
    }
    /**
	 * Obtiene un usuario KPTL por el campo id
	 *
	 * @param int $id
	 * @return mixed
	 */
    public static function query_by_id($id, $comoObjeto=true, $campos=null)
    {
    	global $db;

        $rs = DB_ResultSet::Factory($db->GetExtension());
    	$rs->Set = $db->Execute(self::GetQuery($id, $campos));
   		if(($row = $rs->FetchAssoc()))
	   	{
	   		$row['edad'] = $row['fechanacimiento'] == '0000-00-00' ? false : calcularEdadStr($row['fechanacimiento']);

	   		return $comoObjeto ? self::_row2obj($row) : $row;
	   	}

	   	return null;
    }
	/**
	 * Obtiene un usuario KPTL por el campo user_id
	 *
	 * @param int $user_id
	 * @param bool $comoObjeto
	 * @param string $campos
	 * @return mixed
	 */
    public static function query_by_user_id($user_id, $comoObjeto=true, $campos=null)
    {
    	global $db;

        $rs = DB_ResultSet::Factory($db->GetExtension());
    	$rs->Set = $db->Execute(self::GetQuery(array('u.id'=>$user_id), $campos));
   		if(($row = $rs->FetchAssoc()))
	   	{
	   		$row['edad'] = $row['fechanacimiento'] == '0000-00-00' ? false : calcularEdadStr($row['fechanacimiento']);

	   		return $comoObjeto ? self::_row2obj($row) : $row;
	   	}

	   	return null;
    }
}
