<?php
/**
 * Armando Servillera Torres
 * www.pagina-web-barata.com
 * 2011-06-24
 *
 * José Carlos Cruz Parra
 * programadorphpfreelance.com
 * 2011-07-05
 *
 * SIMPLE WEB PHP EDICION ADMINISTRADA
 *
 * inc/modulos/modulos/admin/functions.inc.php
 *
 */

/**
 * Valida los datos de un registro del modulo
 *
 * @param array $data
 * @return bool
 */
function modulos_validarDatos($data)
{
	$ok = true;

	$requeridos = array(
		'titulo'=>'Título',
		'icono'=>'Icono',
		);

	foreach($requeridos as $key=>$nombre)
	{
		if(!isset($data[$key]) || trim($data[$key]) == '')
		{
			SWPHP::_Msg("Falta el dato: {$nombre}", "error");
			$ok = false;
		}
	}

	return $ok;
}
/**
 * Prepara los datos para registrar/actualizar modulo en la base de datos
 *
 * @param array $data
 * @param int $id
 * @return array
 */
function modulos_prepararDatos($data, $id=0)
{
	global $db;

	$ahora = date('Y-m-d H:i:s');

    //Durante la instalación no hay sesión
    $user_id = isset($_SESSION['user']['id']) ? $_SESSION['user']['id'] : 0;

	if($id == 0)
	{
		//Para insertar (instalando módulo)
		foreach($data as $key=>$value)
		{
			switch($key)
			{
				case "clave":
				case "titulo":
				case "descripcion":
				case "icono":
				case "version":
					$data[$key] = trim($value);
					break;
				case "core":
					$data[$key] = $value ? '1' : '0';
					break;
				default:
					//Cualquier otro dato lo descarto
					unset($data[$key]);
					break;
			}
		}

		$data['fechayhoraregistro'] = $ahora;
		$data['user_idregistro'] = $user_id;

		$data['activo'] = isset($data['core']) && $data['core'] ? '1' : '0';
	}
	else
	{
		//Para actualizar
		foreach($data as $key=>$value)
		{
			switch($key)
			{
				case "titulo":
				case "icono":
					$data[$key] = trim($value);
					break;
				default:
					//Cualquier otro dato lo descarto
					unset($data[$key]);
					break;
			}
		}
	}

	$data['fechayhoraultima'] = $ahora;
	$data['user_idultima'] = $user_id;

	return $data;
}
/**
 * Instala un módulo
 *
 * @param string $clave
 * @return bool
 */
function modulos_instalarModulo($clave)
{
	global $db;

	$clave = trim($clave);

	//Comprobar que se ha indicado una clave
	if(is_string($clave) && strlen($clave) > 0)
	{
		$dir = SWPHP_MODULES_DIR.$clave.'/';

		//Comprobar que la carpeta existe
		if(is_dir($dir))
		{
			$info = SWPHP::_GetModuloInfo($clave);

			//Comprobar que carpeta y clave coinciden
			if($info['clave'] == $clave)
			{
				//Comprobar que no está ya instalado
				if(!$db->Exists($clave, 'clave', SWPHP_DBPREFIX.'modulos'))
				{
					//Comprobar que la versión de SWPHP es soportada por el módulo
					if(isset($info['swphp']) && SWPHP_VERSION >= $info['swphp'])
					{
						$ok = true;

						//Si existe install.php, incluirlo
						$infopath = SWPHP_MODULES_DIR."{$clave}/install.php";
						if(is_file($infopath))
						{
							include($infopath);

							$funcion_instalar = "{$clave}_instalar";

							if(function_exists($funcion_instalar))
							{
								$ok = $funcion_instalar($info);
							}
						}

						if($ok)
						{
							//TODO: ¿validar datos?

							//Registrar el módulo en la tabla modulos
							$info['version'] = '0.0.0'; //Lo registramos como versión mínima para hacer luego las actualizaciones
							if(SWPHP::_ExecuteQuery(
								buildQuery_Insert(modulos_prepararDatos($info), SWPHP_DBPREFIX.'modulos'),
								"Se ha registrado el módulo",
								"No se ha podido registrar el módulo"))
							{
								$modulo_id = $db->Insert_id();
								$perfiles = SWPHP::_GetPerfiles(0);

								$sinErrores = true;

								//Crear los registros en modulos_perfiles, sin permiso para ninguno
								foreach($perfiles as $perfil_id=>$perfil_nombre)
								{
									$sinErrores = $sinErrores && SWPHP::_ExecuteQuery(
										sprintf("INSERT INTO ".SWPHP_DBPREFIX."modulos_perfiles(modulo_id, perfil_id, permiso_id) VALUES(%u, %u, '0')", $modulo_id, $perfil_id),
										"",
										"No se ha podido crear la relación de permiso con el perfil {$perfil_nombre}");
								}

								//El instalador instala el módulo según la versión 1.0 del mismo,
								//así que ahora ejecuto las actualizaciones si las hay
								$sinErrores &= SWPHP::_CheckVersionModulo($clave);

								if($sinErrores)
								{
									//Informar de que ya ha terminado
									SWPHP::_Msg("Instalación realizada", "check");
								}
								else
								{
									//Informar de que ya ha terminado
									SWPHP::_Msg("Instalación realizada con algún error", "aviso");
								}

								return true;
							}
						}
						else
						{
							SWPHP::_Msg("{$funcion_instalar} no se completó correctamente", "aviso");
						}
					}
					else
					{
						SWPHP::_Msg("El módulo es para la versión de SWPHP <b>{$info['swphp']}</b> o superior, pero tu versión es <b>".SWPHP_VERSION."</b>.<br />Tienes que actualizar tu sistema SWPHP para poder instalar el módulo <b>{$info['clave']} {$info['version']}</b>.<br />Si necesitas ayuda, puedes contactar con un experto en <a class=\"external\" href=\"http://simplewebphp.com\">simplewebphp.com</a>", "error");
					}
				}
				else
				{
					SWPHP::_Msg("Ya hay un módulo instalado con esa clave", "error");
				}
			}
			else
			{
				SWPHP::_Msg("El nombre de la carpeta y la clave del módulo no coinciden", "error");
			}
		}
		else
		{
			SWPHP::_Msg("No se encuentra la carpeta del módulo a instalar", "error");
		}
	}
	else
	{
		SWPHP::_Msg("No se ha indicado la clave del módulo a instalar", "error");
	}

	SWPHP::_Msg("No se ha podido completar la instalación", "info");

	return false;
}
/**
 * Reinstala un módulo
 *
 * @param string $clave
 * @return bool
 */
function modulos_reinstalarModulo($clave)
{
	global $db;

	$clave = trim($clave);

	//Comprobar que se ha indicado una clave
	if(is_string($clave) && strlen($clave) > 0)
	{
		$dir = SWPHP_MODULES_DIR.$clave.'/';

		//Comprobar que la carpeta existe
		if(is_dir($dir))
		{
			$info = SWPHP::_GetModuloInfo($clave);

			//Comprobar que carpeta y clave coinciden
			if($info['clave'] == $clave)
			{
				//Comprobar que ya está instalado
				if($db->Exists($clave, 'clave', SWPHP_DBPREFIX.'modulos'))
				{
					$ok = true;

					//Si existe install.php, incluirlo
					$infopath = SWPHP_MODULES_DIR."{$clave}/install.php";
					if(is_file($infopath))
					{
						include($infopath);

						$funcion_instalar = "{$clave}_instalar";

						if(function_exists($funcion_instalar))
						{
							$ok = $funcion_instalar($info);
						}
					}

					if($ok)
					{
						//TODO: ¿validar datos?

						$modulo_id = $db->GetValueQuery(sprintf("SELECT id FROM ".SWPHP_DBPREFIX."modulos WHERE clave = '%s'", addcslashes($clave,"'")), false);
						$perfiles = SWPHP::_GetPerfiles(0);

						//Crear los registros en modulos_perfiles, sin permiso para ninguno
						foreach($perfiles as $perfil_id=>$perfil_nombre)
						{
							SWPHP::_ExecuteQuery(
								sprintf("INSERT INTO ".SWPHP_DBPREFIX."modulos_perfiles(modulo_id, perfil_id, permiso_id) VALUES(%u, %u, '0')", $modulo_id, $perfil_id),
								"",
								"",
								false
								);
						}

						//Informar de que ya ha terminado
						SWPHP::_Msg("Reinstalación realizada", "check");

						return true;
					}
					else
					{
						SWPHP::_Msg("{$funcion_instalar} no se completó correctamente", "aviso");
					}
				}
				else
				{
					SWPHP::_Msg("No hay un módulo instalado con esa clave", "error");
				}
			}
			else
			{
				SWPHP::_Msg("El nombre de la carpeta y la clave del módulo no coinciden", "error");
			}
		}
		else
		{
			SWPHP::_Msg("No se encuentra la carpeta del módulo a instalar", "error");
		}
	}
	else
	{
		SWPHP::_Msg("No se ha indicado la clave del módulo a instalar", "error");
	}

	SWPHP::_Msg("No se ha podido completar la instalación", "info");

	return false;
}
/**
 * Desinstala un módulo
 *
 * @param int $modulo_id
 * @return bool
 */
function modulos_desinstalarModulo($clave)
{
	global $db;

	$clave = trim($clave);

	//Comprobar que se ha indicado una clave
	if(is_string($clave) && strlen($clave) > 0)
	{
		//Comprobar que está instalado
		if($db->Exists($clave, 'clave', SWPHP_DBPREFIX.'modulos'))
		{
			$modulo = $db->GetRow(sprintf("SELECT id, activo, core FROM ".SWPHP_DBPREFIX."modulos WHERE clave = '%s'", addcslashes($clave,"'")));

			//Comprobar que está desactivado
			if(!$modulo['activo'])
			{
				//Comprobar que no es un módulo core
				//(se hace al principio del admin/index.php de este módulo, pero bueno, por si acaso)
				if(!$modulo['core'])
				{
					//Eliminar el registro de la tabla modulos
					if(SWPHP::_ExecuteQuery(
						buildQuery_Delete(SWPHP_DBPREFIX.'modulos', $modulo['id']),
						"Se ha eliminado el registro del módulo",
						"No se ha podido eliminar el registro del módulo"))
					{
						//Eliminar los registros en modulos_perfiles
						$db->Execute("DELETE FROM ".SWPHP_DBPREFIX."modulos_perfiles WHERE modulo_id = '{$modulo['id']}'");

						$ok = true;

						//Si existe install.php, incluirlo
						$infopath = SWPHP_MODULES_DIR."{$clave}/install.php";
						if(is_file($infopath))
						{
							include($infopath);

							$funcion_desinstalar = "{$clave}_desinstalar";

							if(function_exists($funcion_desinstalar))
							{
								$ok = $funcion_desinstalar(SWPHP::_GetModuloInfo($clave));
							}
						}

						if($ok)
						{
							//Informar de que ya ha terminado
							SWPHP::_Msg("Desinstalación realizada", 'check');
						}
						else
						{
							//Informar de que ya ha terminado
							SWPHP::_Msg("Desinstalación realizada con algún error", 'aviso');
						}

						return true;
					}
				}
				else
				{
					SWPHP::_Msg("Se trata de un módulo necesario del sistema, no se puede desinstalar", "error");
				}
			}
			else
			{
				SWPHP::_Msg("El módulo indicado está activado, no se puede desinstalar", "error");
			}
		}
		else
		{
			SWPHP::_Msg("No hay ningún módulo instalado con esa clave", "error");
		}
	}
	else
	{
		SWPHP::_Msg("No se ha indicado la clave del módulo a desinstalar", "error");
	}

	return false;
}
