<?php
/**
 * Jose Carlos Cruz Parra
 * programadorphpfreelance.com
 * 2013-12-28
 *
 * SIMPLE WEB PHP EDICION ADMINISTRADA
 *
 * inc/modulos/traducciones/admin/functions.inc.php
 *
 */

/**
 * Obtiene las frases para traducir
 *
 * @param string $dirPHP
 * @param string $dirTPL
 * @param string $clave
 * @return array
 */
function traducciones_GetFrases($dirPHP, $dirTPL, $clave)
{
	$frases = traducciones_GetFrasesFromPHP($dirPHP, $clave);
	$aux = traducciones_GetFrasesFromTPL($dirTPL, $clave);
	foreach($aux as $frase)
	{
		if(!in_array($frase, $frases))
		{
			$frases[] = $frase;
		}
	}

	return $frases;
}
/**
 * Obtiene las frases para traducir que hay en los archivos php de un directorio
 *
 * @param string $dir
 * @param string $clave
 * @param string $ext
 * @return array
 */
function traducciones_GetFrasesExt($dir, $clave, $ext)
{
	$frases = array();

	if(is_array($dir))
	{
		$func = "traducciones_GetFrasesFrom".mb_strtoupper($ext);
		foreach($dir as $item)
		{
			$aux = $func($item, $clave);
			foreach($aux as $frase)
			{
				if(!in_array($frase, $frases))
				{
					$frases[] = $frase;
				}
			}
		}
	}
	elseif($dir)
	{
		$func = "traducciones_GetFrasesFrom".mb_strtoupper($ext)."file";
		$files = getFilesExt($dir, array($ext), true, true);
		foreach($files as $file)
		{
			$aux = $func($file, $clave);
			foreach($aux as $frase)
			{
				if(!in_array($frase, $frases))
				{
					$frases[] = $frase;
				}
			}
		}
	}

	return $frases;
}
/**
 * Obtiene las frases para traducir que hay en los archivos php de un directorio
 *
 * @param string $dir
 * @param string $clave
 * @return array
 */
function traducciones_GetFrasesFromPHP($dir, $clave)
{
	return traducciones_GetFrasesExt($dir, $clave, 'php');
}
/**
 * Obtiene las frases para traducir que hay en un archivo php
 *
 * @param string $file
 * @param string $clave
 * @return array
 */
function traducciones_GetFrasesFromPHPfile($file, $clave)
{
	$frases = array();

	$str = file_get_contents($file);
	$matches = array();
	preg_match_all('/l\((\'|")((?!(,\s?(\'|")'.$clave.'(\'|"))).)+(\'|"),\s?(\'|")'.$clave.'(\'|")\)/', $str, $matches);

	if(is_array($matches) && is_array($matches[0]) && !empty($matches[0]))
	{
		foreach($matches[0] as $item)
		{
			$aux = mb_substr($item, 2, mb_strrpos($item, ',') - mb_strlen($item));
			$frases[] = eval("return {$aux};");
		}
	}

	return $frases;
}
/**
 * Obtiene las frases para traducir que hay en los archivos tpl de un directorio
 *
 * @param string $dir
 * @param string $clave
 * @return array
 */
function traducciones_GetFrasesFromTPL($dir, $clave)
{
	return traducciones_GetFrasesExt($dir, $clave, 'tpl');
}
/**
 * Obtiene las frases para traducir que hay en un archivo php
 *
 * @param string $file
 * @param string $clave
 * @return array
 */
function traducciones_GetFrasesFromTPLfile($file, $clave)
{
	$frases = array();

	//Si no es un theme entonces es un módulo y tengo que mirar si existe el tpl personalizado en el theme actual
	if($clave != 'theme')
	{
		$aux = SWPHP_THEME_DIR.'tpl/'.str_replace('/tpl/', '/', str_replace(SWPHP_INC_DIR, '', $file));
		if(is_file($aux))
		{
			//Puedo optar por tomar sólo el tpl personalizado, o los dos (prefiero lo segundo)

			$frases = traducciones_GetFrasesFromTPLfile($aux, $clave); //tomar los dos
			//$file = $aux; //tomar sólo el tpl personalizado
		}
	}

	$str = file_get_contents($file);
	$matches = array();
	preg_match_all('/{l\ss=(\'|")((?!((\'|")\sc=(\'|")'.$clave.'(\'|")})).)+(\'|")\sc=(\'|")'.$clave.'(\'|")}/', $str, $matches);

	if(is_array($matches) && !empty($matches) && is_array($matches[0]) && !empty($matches[0]))
	{
		foreach($matches[0] as $item)
		{
			$aux = mb_substr($item, 5, mb_strrpos($item, '=') - mb_strlen($item) - 2);
			$frases[] = eval("return {$aux};");
		}
	}

	return $frases;
}

function traducciones_GetLang($que, $clave, $isocode)
{
	global $swphp_lang;

	//temporalmente
	$aux1 = $swphp_lang;

	$swphp_lang = array(); //Ojo, aquí no hay que poner $swphp_lang global

	if($que == 'theme')
	{
		SWPHP::_Include(SWPHP_THEMES_DIR."{$clave}/lang/{$isocode}/index.php");
	}
	else
	{
		SWPHP::_LoadModuleLang($clave, $isocode);
	}

	//restauro
	$aux2 = $swphp_lang;
	$swphp_lang = $aux1;

	return $aux2;
}
?>